--[[----------------------------------------------------------------------------

Copyright (c) 2007, Timothy Armes (http://www.timothyarmes.com)

All rights reserved.

See included file LICENSE.txt for details.

------------------------------------------------------------------------------]]

local LrPathUtils = import "LrPathUtils"
local LrShell = import "LrShell"
local LrDialogs = import "LrDialogs"
local LrTasks = import "LrTasks"
local LrErrors = import "LrErrors"
local LrFileUtils = import "LrFileUtils"
local LrPathUtils = import "LrPathUtils"
local LrFtp = import "LrFtp"

require "LRMogrifyLogger"
require "LRMogrifyTokens"
require "LRMogrifyBorderSection"
require "LRMogrifyWatermarkSection"

LRMogrifyExportTask = {}

LRMogrifyExportTask.donated = false

function LRMogrifyExportTask.donatationString()
   if LRMogrifyExportTask.donated then
      return LOC "$$$/LRMogrify/DonationReceived=(Unrestricted version)"
   else
      return LOC "$$$/LRMogrify/DonationNotReceived=(Limited version, please donate)"
   end
end

function LRMogrifyExportTask.commandLineForPhoto(opts, photo)

   local resizeCommand = LRMogrifyResizeSection.resizeCommandLine(opts, photo)
   local sharpenCommand = LRMogrifyResizeSection.sharpenCommandLine(opts, photo)
   local watermarkCommand = LRMogrifyWatermarkSection.watermarkCommandLine(opts, photo)
   local textCommand = LRMogrifyWatermarkSection.textCommandLine(opts, photo)
   local borderCommand = LRMogrifyBorderSection.commandLine(opts, photo)
   local colourCommand = LRMogrifyResizeSection.colourCommandLine(opts, photo)
   local formatCommand = LRMogrifyResizeSection.formatCommandLine(opts, photo)

   local commandLine
   if opts.mogrify_text_before then
      if opts.mogrify_watermark_before then
         commandLine = watermarkCommand .. textCommand .. borderCommand
      else
         commandLine = textCommand .. borderCommand .. watermarkCommand
      end
   else
      if opts.mogrify_watermark_before then
         commandLine = watermarkCommand .. borderCommand .. textCommand
      else
         commandLine = borderCommand .. textCommand .. watermarkCommand
      end
   end    

   commandLine = opts.mogrify_command_line .. resizeCommand .. sharpenCommand .. colourCommand .. commandLine .. formatCommand .. " " .. opts.mogrify_command_line_end
   
   return commandLine
   
end

function LRMogrifyExportTask.priorToRender(INFO)
   
   INFO.LOCAL.count = 0
   LRMogrifyExportTask.deleteExported = false
   
   return true -- No additional error
end

function LRMogrifyExportTask.startOfOneRender(INFO)

   local exportContext = INFO.exportContext
   local exportSession = exportContext.exportSession
   local opts          = exportContext.propertyTable
   local photo         = INFO.rendition.photo
   local success       = INFO.success
   local pathOrMessage = INFO.image_path

   if not success then
      return true -- no additional error
   end

   if IS_PIGLET and not opts.mogrify_enabled then
      return true
   end

   INFO.LOCAL.count = INFO.LOCAL.count + 1

   if LRMogrifyExportTask.donated == false and INFO.LOCAL.count >=  10 then
      LrErrors.throwUserError(LOC "$$$/LRMogrify/Donation/MaxFiles=This version of LR/Mogrify can only export up to 10 files.  Please consider offering a donation in order to download the unrestricted version.")
   end

   opts.mogrify_counter = INFO.LOCAL.count
   local commandLine = LRMogrifyExportTask.commandLineForPhoto(opts, photo)
   local pathToMogrify = ''
   
   if WIN_ENV == true or opts.mogrify_usebuiltin == false then
      pathToMogrify = opts.mogrify_path
   else
      pathToMogrify = LrPathUtils.child( _PLUGIN.path, "LRMogrify.mac.mogrify")
   end

   commandLine = commandLine .. ' "' .. pathOrMessage .. '"'
         
   local command = '"' .. pathToMogrify .. '" ' .. commandLine
   local quotedCommand = command
   if WIN_ENV == true then
      -- On Windows, the entire command needs to be wrapped in quotes
      quotedCommand = '"' .. command .. '"'
   end

   -- Run Mogrify
   if LrTasks.execute(quotedCommand) ~= 0 then
      LrDialogs.message(
                        LOC "$$$/LRMogrify/Task/ErrorExe=Failed to run Mogrify, aborting export.",
                        LOC ("$$$/LRMogrify/Task/ErrorExeDetails=Failed to run Mogrify with the following command line:^r^n^1", command),
                        "critical"
                     )
      LrErrors.throwCanceled()
   end
   
   if LRMogrifyExportTask.deleteExported then
   
      INFO.image_path = LrPathUtils.replaceExtension(pathOrMessage, (opts.mogrify_format_type == "jpeg" and "jpg") or opts.mogrify_format_type)

      if LrFileUtils.exists(pathOrMessage) then
      
         -- We've created a new file.  This happen when we either convert the format OR export as tiff (since the .tif is exported as a .tiff).
         -- This is great for us, since mogrify can't read the TIFF's exif data, so the new file isn't yet complete.  Since the original file is 
         -- still available, we copy it over using exiftool. 

         commandline = ' -overwrite_original -tagsfromfile "' .. pathOrMessage ..'" -exif:all --subifd:all "' .. INFO.image_path ..'"'

         if WIN_ENV == false then
            command = '"' .. LrPathUtils.child( _PLUGIN.path, "LRMogrify.exiftool.exe") .. '" ' .. commandline
            quotedCommand = '"' .. command .. '"'
         else
            command = '"' .. LrPathUtils.child( _PLUGIN.path, "LRMogrify.exiftool") .. '" ' .. commandline
            quotedCommand = command   
         end

         -- Run ExifTool
         if LrTasks.execute(quotedCommand) ~= 0 then
            LrDialogs.message(
                              LOC "$$$/LRMogrify/ExifTool/ErrorExe=Failed to run Exiftool, aborting export.",
                              LOC ("$$$/LRMogrify/ExifTool/ErrorExeDetails=Failed to run ExifTool with the following command line:^r^n^1", command),
                              "critical"
                           )
            LrErrors.throwCanceled()
         end

         -- Delete exported file if necessary (new file created by -format parameter)
   
         LrFileUtils.delete(pathOrMessage)
      end
   end

   return true -- no additional error
end

function LRMogrifyExportTask.processRenderedPhotos(functionContext, exportContext)

   local exportSession = exportContext.exportSession
   local opts = exportContext.propertyTable
   local nPhotos = exportSession:countRenditions()

   local titletext
   if not opts.mogrify_ftp then
      titletext = nPhotos > 1
         and LOC ("$$$/LRMogrify/Task/Progress=Exporting and mogrifying ^1 photos", nPhotos)
      or  LOC ("$$$/LRMogrify/Task/Progress/One=Exporting and mogrifying one photo")
   else
      titletext = nPhotos > 1
         and LOC ("$$$/LRMogrify/Task/ProgressFTP=Exporting, mogrifying and FTPing ^1 photos", nPhotos)
      or  LOC ("$$$/LRMogrify/Task/ProgressFTP/One=Exporting, mogrifying and FTPing one photo")
   end

   local progressScope = exportContext:configureProgress { title = titletext }

   -- Create an FTP connection

   local ftp_failures = {}
   local ftpInstance = nil

   if opts.mogrify_ftp then
   
      ftpInstance = LrFtp.create( opts.mogrify_ftp_preset )
      
      if not ftpInstance then
         LrErrors.throwUserError( LOC "$$$/FtpUpload/Upload/Errors/InvalidFtpParameters=The specified FTP preset is incomplete and cannot be used." )
      end
      
      -- Ensure target directory exists
      local index = 0
      while true do
         
         local subPath = string.sub( opts.mogrify_ftp_full_path, 0, index )
         ftpInstance.path = subPath
         
         local exists, message = ftpInstance:exists( '' )
         
         if exists == false then
            local success = ftpInstance:makeDirectory( '' )
            
            if not success then
               -- Possible situation if permissions don't allow us to create directories
               LrErrors.throwUserError(LOC "$$$/FtpUpload/Upload/Errors/CannotMakeDirectoryForUpload=Cannot upload because Lightroom could not create the destination directory.")
            end
            
         elseif exists == 'file' then
            -- Unlikely, due to the ambiguous way paths for directories get tossed around
            LrErrors.throwUserError(LOC "$$$/FtpUpload/Upload/Errors/UploadDestinationIsAFile=Cannot upload to a destination that already exists as a file.")
         elseif exists == 'directory' then
            -- Excellent, it exists, do nothing here
         else
            -- Not sure if this would every really happen
            LrErrors.throwUserError(LOC "$$$/FtpUpload/Upload/Errors/CannotCheckForDestination=Unable to upload because Lightroom cannot ascertain if the target destination exists.")
         end
         
         if index == nil then
            break
         end
         
         index = string.find(opts.mogrify_ftp_full_path, "/", index + 1)
         
      end
      
      ftpInstance.path = opts.mogrify_ftp_full_path
   end

   -- Process each file in turn

   local INFO = {
      exportContext = exportContext,
      LOCAL = {},
   };

   LRMogrifyExportTask.priorToRender(INFO)

   for i, rendition in exportContext:renditions { stopIfCanceled = true } do
      
      local photo = rendition.photo
      local success, pathOrMessage = rendition:waitForRender()

      if progressScope:isCanceled() then break end

      INFO.photo = photo
      INFO.success = success
      INFO.image_path = pathOrMessage
      INFO.rendition = rendition
      LRMogrifyExportTask.startOfOneRender(INFO)

      if INFO.success then
         -- FTP the file if needed
         
         if opts.mogrify_ftp then
            local newPath
            
            if LRMogrifyExportTask.deleteExported then
               -- File has been renamed
               newPath = LrPathUtils.replaceExtension(pathOrMessage, (opts.mogrify_format_type == "jpeg" and "jpg") or opts.mogrify_format_type)	
            else
               newPath = pathOrMessage
            end

            local filename = LrPathUtils.leafName(newPath)		
            local success, errorMessage = ftpInstance:putFile(newPath, filename)

            if not success then
               -- if we can't upload that file, log it.  For example, maybe user has exceeded disk
               -- quota, or the file already exists and we don't have permission to overwrite, or
               -- we don't have permission to write to that directory, etc....
               table.insert(ftp_failures, filename)
            end

            -- When done with photo, delete temp file. There is a cleanup step that happens later,
            -- but this will help manage space in the event of a large upload.
            
            if opts.mogrify_ftp_delete then
               LrFileUtils.delete(newPath)
            end
         end
         
      else
         LrErrors.throwUserError(pathOrMessage)
      end

   end

   if opts.mogrify_ftp then
      ftpInstance:disconnect()
      
      if #ftp_failures > 0 then
         local message
         if #ftp_failures == 1 then
            message = LOC "$$$/FtpUpload/Upload/Errors/OneFileFailed=1 file failed to upload correctly."
         else
            message = LOC ("$$$/FtpUpload/Upload/Errors/SomeFileFailed=^1 files failed to upload correctly.", #ftp_failures)
         end
         LrDialogs.message(message, table.concat( ftp_failures, "\n" ))
      end
   end
end
